@@ -1,12 +1,12 @@
 /* mpf_set_z -- Assign a float from an integer.
 
-Copyright (C) 1996 Free Software Foundation, Inc.
+Copyright 1996, 2001, 2004 Free Software Foundation, Inc.
 
 This file is part of the GNU MP Library.
 
 The GNU MP Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The GNU MP Library is distributed in the hope that it will be useful, but
@@ -15,21 +15,13 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MP Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+along with the GNU MP Library.  If not, see http://www.gnu.org/licenses/.  */
 
 #include "gmp.h"
 #include "gmp-impl.h"
 
 void
-#if __STDC__
 mpf_set_z (mpf_ptr r, mpz_srcptr u)
-#else
-mpf_set_z (r, u)
-     mpf_ptr r;
-     mpz_srcptr u;
-#endif
 {
   mp_ptr rp, up;
   mp_size_t size, asize;
@@ -49,6 +41,6 @@ mpf_set_z (r, u)
       asize = prec;
     }
 
-  MPN_COPY (rp, up, asize);
   SIZ (r) = size >= 0 ? asize : -asize;
+  MPN_COPY (rp, up, asize);
 }
