@@ -1,13 +1,13 @@
 /* spect.c -- the spectral test */
 
 /*
-Copyright (C) 1999 Free Software Foundation, Inc.
+Copyright 1999 Free Software Foundation, Inc.
 
 This file is part of the GNU MP Library.
 
 The GNU MP Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The GNU MP Library is distributed in the hope that it will be useful, but
@@ -16,9 +16,7 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MP Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA.
+along with the GNU MP Library.  If not, see http://www.gnu.org/licenses/.  */
 */
 
 /* T is upper dimension.  Z_A is the LC multiplier, which is
@@ -129,7 +127,7 @@ main (int argc, char *argv[])
 }
 
 
-void 
+void
 debug_foo()
 {
   if (0)
